using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Hosting;
using NetworkResurrector.Agent.Extensions;
using NetworkResurrector.Agent.Extensions.Serilog;
using Serilog;
using Serilog.Core;
using Serilog.Events;
using Serilog.Sinks.MSSqlServer;
using System;

namespace NetworkResurrector.Agent
{
    public class Program
    {
        public static void Main(string[] args)
        {
            var builder = WebApplication.CreateBuilder(args);

            builder.Host.UseSerilog((_, lc) =>
            {
                var connectionString = builder.Configuration.GetConnectionString("DatabaseConnection");
                var loggingLevelParam = builder.Configuration.GetValue<string>("Logging:LogLevel:Default");

                var loggingLevelOk = Enum.TryParse(loggingLevelParam, out LogEventLevel loggingLevel);
                if (!loggingLevelOk)
                    throw new Exception($"Logging level '{loggingLevelParam}' is not valid.");

                var loggingLevelSwitch = new LoggingLevelSwitch(loggingLevel);

                var columnOptions = new ColumnOptions();
                columnOptions.Store.Remove(StandardColumn.Properties);
                columnOptions.Store.Remove(StandardColumn.MessageTemplate);
                columnOptions.Store.Add(StandardColumn.LogEvent);
                var mssqlSinkOptions = new MSSqlServerSinkOptions() { AutoCreateSqlTable = true, TableName = "__Logs_Agent" };

                lc
                .MinimumLevel.ControlledBy(loggingLevelSwitch)
                .MinimumLevel.Override("Microsoft", LogEventLevel.Warning)
                .Enrich.FromLogContext()
                .Enrich.With(new AgentCodeEventEnricher(builder.Configuration))
                .WriteTo.Console()
                .WriteTo.MSSqlServer(connectionString, mssqlSinkOptions, columnOptions: columnOptions);
            });

            builder.Services.ConfigureServices(builder.Configuration);

            var useWindowsService = builder.Configuration.GetValue<bool>("Host:UseWindowsService");
            if (useWindowsService)
                builder.Host.UseWindowsService();

            var app = builder.Build();
            app.Configure(builder.Configuration);

            var exitCode = 0;
            try
            {
                var urls = builder.Configuration.GetValue<string>("Urls");
                var agentCode = builder.Configuration.GetValue<string>("Agent:Code");

                Log.Information($"Starting network resurrector agent {agentCode}...");
                Log.Information($"Network resurrector agent {agentCode} listening on {urls}");
                Log.Information($"Hostname: {Environment.MachineName}");
                Console.WriteLine("Application started. Press Ctrl+C to shut down.");
                app.Run();
            }
            catch (Exception ex)
            {
                Log.Fatal(ex, "Network resurrector agent host terminated unexpectedly");
                exitCode = 1;
            }
            finally
            {
                Log.CloseAndFlush();
            }
            Environment.Exit(exitCode);
        }
    }
}
